/*--------------------------------------------------------------------------+
$Id$
|                                                                          |
| Copyright 2012 ForTISS GmbH                     |
|                                                                          |
| Licensed under the Apache License, Version 2.0 (the "License");          |
| you may not use this file except in compliance with the License.         |
| You may obtain a copy of the License at                                  |
|                                                                          |
|    http://www.apache.org/licenses/LICENSE-2.0                            |
|                                                                          |
| Unless required by applicable law or agreed to in writing, software      |
| distributed under the License is distributed on an "AS IS" BASIS,        |
| WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. |
| See the License for the specific language governing permissions and      |
| limitations under the License.                                           |
+--------------------------------------------------------------------------*/
package org.fortiss.tooling.graphicsGL.ui.objects;

import static org.fortiss.tooling.graphicsGL.ui.util.GLPrimitives.drawLine;
import static org.fortiss.tooling.graphicsGL.ui.util.GLPrimitives.flushName;
import static org.fortiss.tooling.graphicsGL.ui.util.GLPrimitives.moveTo;
import static org.fortiss.tooling.graphicsGL.ui.util.GLPrimitives.prepareName;
import static org.fortiss.tooling.graphicsGL.ui.util.GLPrimitives.restoreMatrix;
import static org.fortiss.tooling.graphicsGL.ui.util.GLPrimitives.saveMatrix;
import static org.fortiss.tooling.graphicsGL.ui.util.GLPrimitives.setColor;
import static org.fortiss.tooling.graphicsGL.util.Graphics3DModelElementFactory.RED;
import static org.fortiss.tooling.graphicsGL.util.Graphics3DModelElementFactory.WHITE;
import static org.lwjgl.util.glu.GLU.GLU_FILL;

import org.fortiss.tooling.graphicsGL.model.Color3D;
import org.fortiss.tooling.graphicsGL.model.graph.Axis3D;
import org.fortiss.tooling.graphicsGL.model.graph.CoordinateSystem3D;
import org.fortiss.tooling.graphicsGL.model.graph.Graph3D;
import org.fortiss.tooling.graphicsGL.model.graph.GraphLine3D;
import org.fortiss.tooling.graphicsGL.model.graph.GraphPoint3D;
import org.fortiss.tooling.graphicsGL.ui.picker.ModelObjectPicker;
import org.lwjgl.util.glu.Sphere;

/**
 * View object for {@link Graph3D}.
 * 
 * @author hoelzl
 * @author $Author$
 * @version $Rev$
 * @ConQAT.Rating YELLOW Hash: E6B8826D13F12C42C0B5170FB2BBD32C
 */
public class Graph3DViewObject extends ViewObjectBase {
	/** The graph model. */
	private final Graph3D graph;

	/** The highlighted model object. */
	private GraphPoint3D highlighted;

	/** Constructor. */
	public Graph3DViewObject(Graph3D graph) {
		this.graph = graph;
	}

	/** Sets the currently highlighted point. */
	public void setHighlightedPoint(GraphPoint3D point) {
		highlighted = point;
	}

	/** {@inheritDoc} */
	@Override
	public void renderObject() {
		renderCoordSys(graph.getCoordinateSystem());
		setColor(WHITE);
		for(GraphPoint3D p : graph.getPointsList()) {
			if(p == highlighted) {
				setColor(RED);
				renderPoint(p);
				setColor(WHITE);
			} else {
				renderPoint(p);
			}
		}
		for(GraphLine3D l : graph.getLinesList()) {
			drawLine(l.getStart().getPosition(), l.getEnd().getPosition());
		}
	}

	/** {@inheritDoc} */
	@Override
	public void renderObjectForPicking(ModelObjectPicker picker) {
		// only points are pickable
		for(GraphPoint3D p : graph.getPointsList()) {
			prepareName(p, picker);
			renderPoint(p);
			flushName();
		}
	}

	/** The GLU sphere used for {@link GraphPoint3D} rendering. */
	private static Sphere SPHERE = new Sphere();

	static {
		SPHERE.setDrawStyle(GLU_FILL);
	}

	/** Renders the given point. */
	private void renderPoint(GraphPoint3D p) {
		saveMatrix();
		moveTo(p.getPosition());
		SPHERE.draw(0.2f, 9, 4);
		restoreMatrix();
	}

	/** Renders the coordinate system if present. */
	private void renderCoordSys(CoordinateSystem3D coordinateSystem) {
		if(coordinateSystem != null) {
			if(coordinateSystem.getXAxis() != null) {
				Axis3D a = coordinateSystem.getXAxis();
				renderAxis(a.getColor(), a.getLength(), 0, 0);
			}
			if(coordinateSystem.getYAxis() != null) {
				Axis3D a = coordinateSystem.getYAxis();
				renderAxis(a.getColor(), 0, a.getLength(), 0);
			}
			if(coordinateSystem.getZAxis() != null) {
				Axis3D a = coordinateSystem.getZAxis();
				renderAxis(a.getColor(), 0, 0, a.getLength());
			}
		}
	}

	/** Renders the axis. */
	private void renderAxis(Color3D color, float x, float y, float z) {
		setColor(color);
		drawLine(0, 0, 0, x, y, z);
	}
}
